// DIA

// Main entry point for random map script
void main(void)
{

   // Text
   rmSetStatusText("",0.01);

   // Set size.
   int playerTiles=12000;
   if(cMapSize == 1)
   {
      playerTiles = 15600;
      rmEchoInfo("Large map");
   }

   int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles);
   rmEchoInfo("Map size="+size+"m x "+size+"m");
   rmSetMapSize(size, size);

   // Set up default water.
   rmSetSeaLevel(0.0);
   rmSetSeaType("Aegean Sea");

   // Init map.
   rmTerrainInitialize("water");

   // Define some classes.
   int classPlayer=rmDefineClass("player");
   rmDefineClass("starting settlement");
   int classBonusHuntable=rmDefineClass("bonus huntable");
   rmDefineClass("classHill");

   // Grow a big continent.
   int centerID=rmCreateArea("center");
   rmSetAreaSize(centerID, 0.32, 0.28);
   rmSetAreaLocation(centerID, 0.5, 0.5);
   /*
   rmSetAreaMinBlobs(centerID, 3);
   rmSetAreaMaxBlobs(centerID, 6);
   rmSetAreaMinBlobDistance(centerID, 20.0);
   rmSetAreaMaxBlobDistance(centerID, 40.0);
   */
   rmSetAreaCoherence(centerID, 0.0);
   rmSetAreaBaseHeight(centerID, 2.0);
   rmSetAreaTerrainType(centerID, "SandDirt50b");
   rmSetAreaSmoothDistance(centerID, 30);
   rmSetAreaHeightBlend(centerID, 2);
   rmAddAreaConstraint(centerID, rmCreateBoxConstraint("center-edge", 0.05, 0.05, 0.95, 0.95, 0.01));
   rmBuildArea(centerID);

    // -------------Define constraints

   // Player area constraint.
   int playerConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, 5.0);

   // Create a edge of map constraint.
   int edgeConstraint=rmCreateBoxConstraint("edge of map", rmXTilesToFraction(20), rmZTilesToFraction(20), 1.0-rmXTilesToFraction(20), 1.0-rmZTilesToFraction(20), 0.01);

   // Settlement constraint.
   int shortAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by short distance", "AbstractSettlement", 20.0);
   int farAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by long distance", "AbstractSettlement", 50.0);
   int farStartingSettleConstraint=rmCreateClassDistanceConstraint("objects avoid player TCs", rmClassID("starting settlement"), 40.0);

   // Tower constraint.
   int avoidTower=rmCreateTypeDistanceConstraint("towers avoid towers", "tower", 24.0);

   // Gold
   int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 30.0);
   int shortAvoidGold=rmCreateTypeDistanceConstraint("short avoid gold", "gold", 10.0);

   // Herd animals
   int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 20.0);
   int avoidFood=rmCreateTypeDistanceConstraint("avoid food", "food", 6.0);
   int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 20.0);

   //Hunted animals
   int avoidBonusHuntable=rmCreateClassDistanceConstraint("avoid bonus huntable", classBonusHuntable, 30.0);

   // Avoid impassable land
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "land", false, 10.0);
   int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("short avoid impassable land", "land", false, 5.0);
   int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 15.0);
   int shortHillConstraint=rmCreateClassDistanceConstraint("patches vs. hill", rmClassID("classHill"), 10.0);
   
   int rockaway=rmCreateClassDistanceConstraint("away from rocks", rmClassID("id2"), 40.0);





    // Stay near shore
   int nearShore=rmCreateTerrainMaxDistanceConstraint("near shore", "water", true, 6.0);

   // -------------Define objects
   // Close Objects

   int startingSettlementID=rmCreateObjectDef("Starting settlement");
   rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
   rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
   rmSetObjectDefMinDistance(startingSettlementID, 0.0);
   rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

   // towers avoid other towers
   int startingTowerID=rmCreateObjectDef("Starting tower");
   rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
   rmSetObjectDefMinDistance(startingTowerID, 22.0);
   rmSetObjectDefMaxDistance(startingTowerID, 28.0);
   rmAddObjectDefConstraint(startingTowerID, avoidTower);

   // gold avoids gold
   int startingGoldID=rmCreateObjectDef("Starting gold");
   rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
   rmSetObjectDefMinDistance(startingGoldID, 20.0);
   rmSetObjectDefMaxDistance(startingGoldID, 25.0);
   rmAddObjectDefConstraint(startingGoldID, avoidGold);
   rmAddObjectDefConstraint(startingGoldID, avoidImpassableLand);
   
   int startingGoldID2=rmCreateObjectDef("Starting gold 2");
   rmAddObjectDefItem(startingGoldID2, "Gold mine tiny", 1, 0.0);
   rmSetObjectDefMinDistance(startingGoldID2, 20.0);
   rmSetObjectDefMaxDistance(startingGoldID2, 25.0);
   rmAddObjectDefConstraint(startingGoldID2, avoidGold);
   rmAddObjectDefConstraint(startingGoldID2, avoidImpassableLand);

   int closeCowsID=rmCreateObjectDef("close Cows");
   rmAddObjectDefItem(closeCowsID, "goat", rmRandInt(0,1), 0.0);
   rmSetObjectDefMinDistance(closeCowsID, 25.0);
   rmSetObjectDefMaxDistance(closeCowsID, 30.0);
   rmAddObjectDefConstraint(closeCowsID, avoidImpassableLand);
   rmAddObjectDefConstraint(closeCowsID, avoidFood);

   int closeBerriesID=rmCreateObjectDef("close berries");
   rmAddObjectDefItem(closeBerriesID, "berry bush", rmRandInt(6,9), 4.0);
   rmSetObjectDefMinDistance(closeBerriesID, 20.0);
   rmSetObjectDefMaxDistance(closeBerriesID, 25.0);
   rmAddObjectDefConstraint(closeBerriesID, avoidImpassableLand);
   rmAddObjectDefConstraint(closeBerriesID, avoidFood);
   
   int closeBoarID=rmCreateObjectDef("close Boar");
   float boarNumber=rmRandFloat(0, 1);
   if(boarNumber<0.3)
      rmAddObjectDefItem(closeBoarID, "baboon", 2, 4.0);
   else 
      rmAddObjectDefItem(closeBoarID, "baboon", 1, 4.0);
   rmSetObjectDefMinDistance(closeBoarID, 30.0);
   rmSetObjectDefMaxDistance(closeBoarID, 50.0);
   rmAddObjectDefConstraint(closeBoarID, avoidImpassableLand);
   
   int stragglerTreeID=rmCreateObjectDef("straggler tree");
   rmAddObjectDefItem(stragglerTreeID, "palm", 1, 0.0);
   rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
   rmSetObjectDefMaxDistance(stragglerTreeID, 15.0);

    // player fish
   int fishVsFishID=rmCreateTypeDistanceConstraint("fish v fish", "fish", 10.0);
   int fishLand = rmCreateTerrainDistanceConstraint("fish land", "land", true, 6.0);
   int fishFurther = rmCreateTerrainDistanceConstraint("fish far", "far", true, 12.0);

   int playerFishID=rmCreateObjectDef("owned fish");
   rmAddObjectDefItem(playerFishID, "fish - salmon", 3, 10.0);
   rmSetObjectDefMinDistance(playerFishID, 0.0);
   rmSetObjectDefMaxDistance(playerFishID, 100.0);
   rmAddObjectDefConstraint(playerFishID, fishVsFishID);
   rmAddObjectDefConstraint(playerFishID, fishLand);

   // Medium Objects

   // Text
   rmSetStatusText("",0.10);

   // gold avoids gold and Settlements
   int mediumGoldID=rmCreateObjectDef("medium gold");
   rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(mediumGoldID, 40.0);
   rmSetObjectDefMaxDistance(mediumGoldID, 60.0);
   rmAddObjectDefConstraint(mediumGoldID, avoidGold);
   rmAddObjectDefConstraint(mediumGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(mediumGoldID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumGoldID, farStartingSettleConstraint);
   
   int mediumGoldID2=rmCreateObjectDef("medium gold 2");
   rmAddObjectDefItem(mediumGoldID2, "Gold mine small", 1, 0.0);
   rmSetObjectDefMinDistance(mediumGoldID2, 40.0);
   rmSetObjectDefMaxDistance(mediumGoldID2, 60.0);
   rmAddObjectDefConstraint(mediumGoldID2, avoidGold);
   rmAddObjectDefConstraint(mediumGoldID2, shortAvoidSettlement);
   rmAddObjectDefConstraint(mediumGoldID2, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumGoldID2, farStartingSettleConstraint);

   int numHuntable=rmRandInt(4, 8);

   int mediumDeerID=rmCreateObjectDef("medium deer");
   rmAddObjectDefItem(mediumDeerID, "baboon", numHuntable, 3.0);
   rmSetObjectDefMinDistance(mediumDeerID, 60.0);
   rmSetObjectDefMaxDistance(mediumDeerID, 80.0);
   rmAddObjectDefConstraint(mediumDeerID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumDeerID, avoidFood);
   rmAddObjectDefConstraint(mediumDeerID, farStartingSettleConstraint);

   // Far Objects

   // gold avoids gold, Settlements and TCs
   int farGoldID=rmCreateObjectDef("far gold");
   rmAddObjectDefItem(farGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(farGoldID, 80.0);
   rmSetObjectDefMaxDistance(farGoldID, 150.0);
   rmAddObjectDefConstraint(farGoldID, avoidGold);
   rmAddObjectDefConstraint(farGoldID, avoidImpassableLand);
   rmAddObjectDefConstraint(farGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(farGoldID, farStartingSettleConstraint);
  
   int farCowID=rmCreateObjectDef("far Cow");
   rmAddObjectDefItem(farCowID, "cow", rmRandInt(1,3), 4.0);
   rmSetObjectDefMinDistance(farCowID, 80.0);
   rmSetObjectDefMaxDistance(farCowID, 150.0);
   rmAddObjectDefConstraint(farCowID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farCowID, avoidImpassableLand);
   rmAddObjectDefConstraint(farCowID, avoidHerdable);

   // pick wolves or bears as predators
   // avoid TCs and other animals
   int farPredatorID=rmCreateObjectDef("far predator");
   float predatorSpecies=rmRandFloat(0, 1);
   if(predatorSpecies<0.5)   
      rmAddObjectDefItem(farPredatorID, "baboon", 2, 2.0);
   else
      rmAddObjectDefItem(farPredatorID, "baboon", 1, 1.0);
   rmSetObjectDefMinDistance(farPredatorID, 50.0);
   rmSetObjectDefMaxDistance(farPredatorID, 100.0);
   rmAddObjectDefConstraint(farPredatorID, avoidPredator);
   rmAddObjectDefConstraint(farPredatorID, farStartingSettleConstraint); 
   rmAddObjectDefConstraint(farPredatorID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(farPredatorID, avoidFood);

   // This map will either use boar or deer as the extra huntable food.

   // hunted avoids hunted and TCs
   int bonusHuntableID=rmCreateObjectDef("bonus huntable");
   int bonusChance=rmRandFloat(0, 1);
   if(bonusChance<0.5)   
      rmAddObjectDefItem(bonusHuntableID, "baboon", rmRandInt(4,9), 4.0);
   else
      rmAddObjectDefItem(bonusHuntableID, "baboon", rmRandInt(4,9), 4.0);
   rmSetObjectDefMinDistance(bonusHuntableID, 0.0);
   rmSetObjectDefMaxDistance(bonusHuntableID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(bonusHuntableID, avoidBonusHuntable);
   rmAddObjectDefToClass(bonusHuntableID, classBonusHuntable);
   rmAddObjectDefConstraint(bonusHuntableID, farStartingSettleConstraint); 
   rmAddObjectDefConstraint(bonusHuntableID, avoidImpassableLand);
   
   int randomTreeID=rmCreateObjectDef("random tree");
   rmAddObjectDefItem(randomTreeID, "palm", 1, 0.0);
   rmSetObjectDefMinDistance(randomTreeID, 0.0);
   rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 4.0));
   rmAddObjectDefConstraint(randomTreeID, shortAvoidSettlement);
   rmAddObjectDefConstraint(randomTreeID, avoidImpassableLand);

   // Birds
   int farhawkID=rmCreateObjectDef("far hawks");
   rmAddObjectDefItem(farhawkID, "hawk", 1, 0.0);
   rmSetObjectDefMinDistance(farhawkID, 0.0);
   rmSetObjectDefMaxDistance(farhawkID, rmXFractionToMeters(0.5));

   // Relics avoid TCs
   int relicID=rmCreateObjectDef("relic");
   rmAddObjectDefItem(relicID, "relic", 1, 0.0);
   rmSetObjectDefMinDistance(relicID, 60.0);
   rmSetObjectDefMaxDistance(relicID, 150.0);
   rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 70.0));
   rmAddObjectDefConstraint(relicID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(relicID, shortAvoidImpassableLand);

   // -------------Done defining objects

   // Cheesy "circular" placement of players.
   if(cNumberNonGaiaPlayers > 8)
      rmPlacePlayersCircular(0.2, 0.25, rmDegreesToRadians(5.0));
   else
      rmPlacePlayersCircular(0.25, 0.2, rmDegreesToRadians(5.0));

   // Set up player areas.
   float playerFraction=rmAreaTilesToFraction(3000);
   for(i=1; <cNumberPlayers)
   {
      // Create the area.
      int id=rmCreateArea("Player"+i, centerID);
      rmSetPlayerArea(i, id);
      rmSetAreaSize(id, 0.9*playerFraction, 1.1*playerFraction);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaWarnFailure(id, false);
      rmSetAreaMinBlobs(id, 3);
      rmSetAreaMaxBlobs(id, 6);
      rmSetAreaMinBlobDistance(id, 26.0);
      rmSetAreaMaxBlobDistance(id, 50.0);
      rmSetAreaCoherence(id, 0.0);
      rmAddAreaConstraint(id, playerConstraint);
      rmSetAreaLocPlayer(id, i);
      rmSetAreaTerrainType(id, "GrassDirt50");
      rmAddAreaTerrainLayer(id, "GrassDirt75", 0, 12);

   }

   // Text
   rmSetStatusText("",0.20);

   // Build the areas.
   rmBuildAllAreas();

   for(i=1; <cNumberPlayers*15)
   {
      // Beautification sub area.
      int id2=rmCreateArea("Patch"+i);
      rmSetAreaSize(id2, rmAreaTilesToFraction(10), rmAreaTilesToFraction(20));
      rmSetAreaTerrainType(id2, "CliffGreekB");
      rmSetAreaWarnFailure(id2, false);
      rmSetAreaMinBlobs(id2, 1);
      rmSetAreaMaxBlobs(id2, 2);
      rmSetAreaMinBlobDistance(id2, 3.0);
      rmSetAreaMaxBlobDistance(id2, 5.0);
      rmSetAreaCoherence(id2, 0.0);
      rmAddAreaConstraint(id2, playerConstraint);
      rmAddAreaConstraint(id2, avoidImpassableLand);
      rmBuildArea(id2);
   }

      // Text
   rmSetStatusText("",0.40);

   // Place starting settlements.
   // Close things....
   // TC
   rmPlaceObjectDefPerPlayer(startingSettlementID, true);

   // Towers.
   rmPlaceObjectDefPerPlayer(startingTowerID, true, 4);

   // Elev.
   int numTries=3*cNumberNonGaiaPlayers;
   int failCount=0;
   for(i=0; <numTries)
   {
      int elevID=rmCreateArea("elev"+i, centerID);
      rmSetAreaSize(elevID, rmAreaTilesToFraction(15), rmAreaTilesToFraction(40));
      rmSetAreaWarnFailure(elevID, false);
      rmAddAreaConstraint(elevID, playerConstraint);
      rmAddAreaToClass(elevID, rmClassID("classHill"));
      rmAddAreaConstraint(elevID, shortHillConstraint);
      rmAddAreaConstraint(elevID, avoidImpassableLand);
      if(rmRandFloat(0.0, 1.0)<0.5)
         rmSetAreaTerrainType(elevID, "ShorelineAegeanC");
      rmSetAreaBaseHeight(elevID, rmRandFloat(1.5, 3.0));
      rmSetAreaHeightBlend(elevID, 2);
      rmSetAreaMinBlobs(elevID, 1);
      rmSetAreaMaxBlobs(elevID, 5);
      rmSetAreaMinBlobDistance(elevID, 16.0);
      rmSetAreaMaxBlobDistance(elevID, 30.0);
      rmSetAreaCoherence(elevID, 0.0);
      rmSetAreaSmoothDistance(elevID, 68);

      if(rmBuildArea(elevID)==false)
      {
         // Stop trying once we fail 6 times in a row.
         failCount++;
         if(failCount==6)
            break;
      }
      else
         failCount=0;
   }

   // Slight Elevation
   numTries=10*cNumberNonGaiaPlayers;
   failCount=0;
   for(i=0; <numTries)
   {
      elevID=rmCreateArea("wrinkle"+i, centerID);
      rmSetAreaSize(elevID, rmAreaTilesToFraction(15), rmAreaTilesToFraction(120));
      rmSetAreaWarnFailure(elevID, false);
      rmAddAreaConstraint(elevID, shortAvoidImpassableLand);
      rmAddAreaConstraint(elevID, shortAvoidSettlement);
      rmAddAreaConstraint(elevID, shortHillConstraint);
      rmSetAreaBaseHeight(elevID, rmRandFloat(3.0, 5.0));
      rmSetAreaHeightBlend(elevID, 1);
      rmSetAreaMinBlobs(elevID, 1);
      rmSetAreaMaxBlobs(elevID, 3);
      rmSetAreaMinBlobDistance(elevID, 16.0);
      rmSetAreaMaxBlobDistance(elevID, 20.0);
      rmSetAreaCoherence(elevID, 0.0);

      if(rmBuildArea(elevID)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;
   }

   // Settlements.
   id=rmAddFairLoc("Settlement", false, true,  60, 80, 40, 40); /*back inside */
   rmAddFairLocConstraint(id, avoidImpassableLand);

   if(rmRandFloat(0,1)<0.75)
   {      
      id=rmAddFairLoc("Settlement", true, true, 90, 120, 90, 50);
      rmAddFairLocConstraint(id, avoidImpassableLand);
   }
   else
   {  
      id=rmAddFairLoc("Settlement", false, true,  60, 100, 60, 50);
      rmAddFairLocConstraint(id, avoidImpassableLand);
   }

   if(rmPlaceFairLocs())
   {
      id=rmCreateObjectDef("far settlement2");
      rmAddObjectDefItem(id, "Settlement", 1, 0.0);
      for(i=1; <cNumberPlayers)
      {
         for(j=0; <rmGetNumberFairLocs(i))
            rmPlaceObjectDefAtLoc(id, i, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j), 1);
      }
   }


   // Straggler trees.
   rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(2, 6));

   // Gold
   rmPlaceObjectDefPerPlayer(startingGoldID, false);
   if(rmRandFloat(0, 1) < 0.9)
	rmPlaceObjectDefPerPlayer(startingGoldID2, false);

   // Text
   rmSetStatusText("",0.60);

   // cows
   rmPlaceObjectDefPerPlayer(closeCowsID, true);

   // Berries  
   rmPlaceObjectDefPerPlayer(closeBerriesID, true);

   // Boar.
   bonusChance=rmRandFloat(0, 1);
   if(bonusChance<0.7)
   rmPlaceObjectDefPerPlayer(closeBoarID, false);
   
   // Medium things....
   // Gold
   rmPlaceObjectDefPerPlayer(mediumGoldID, false, 2);
   rmPlaceObjectDefPerPlayer(mediumGoldID2, false, 1);

   // player fish
   rmPlaceObjectDefPerPlayer(playerFishID, false);

   rmPlaceObjectDefPerPlayer(mediumDeerID, false, 1);
            
   // Far things.

   // Gold.
   rmPlaceObjectDefPerPlayer(farGoldID, false, rmRandInt(1, 3));

   // Relics.
   rmPlaceObjectDefPerPlayer(relicID, false);

   // Hawks
   rmPlaceObjectDefPerPlayer(farhawkID, false, 2);

   // Bonus huntable.
   rmPlaceObjectDefPerPlayer(bonusHuntableID, false, 1);

   // Herds.
   rmPlaceObjectDefPerPlayer(farCowID, false, rmRandInt(1,2));

   // Predators
   rmPlaceObjectDefPerPlayer(farPredatorID, false, 1);
   
   // Text
   rmSetStatusText("",0.80);

   // Forest.
   int classForest=rmDefineClass("forest");
   int forestObjConstraint=rmCreateTypeDistanceConstraint("forest obj", "all", 6.0);
   int forestConstraint=rmCreateClassDistanceConstraint("forest v forest", rmClassID("forest"), 20.0);
   int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 20.0);
   int count=0;
   numTries=8*cNumberNonGaiaPlayers;
   failCount=0;
   for(i=0; <numTries)
   {
      int forestID=rmCreateArea("forest"+i, centerID);
      rmSetAreaSize(forestID, rmAreaTilesToFraction(120), rmAreaTilesToFraction(180));
      rmSetAreaWarnFailure(forestID, false);
      rmSetAreaForestType(forestID, "mixed palm forest");
      rmAddAreaConstraint(forestID, forestSettleConstraint);
      rmAddAreaConstraint(forestID, forestObjConstraint);
      rmAddAreaConstraint(forestID, forestConstraint);
      rmAddAreaConstraint(forestID, avoidImpassableLand);
      rmAddAreaToClass(forestID, classForest);
      
      rmSetAreaMinBlobs(forestID, 6);
      rmSetAreaMaxBlobs(forestID, 9);
      rmSetAreaMinBlobDistance(forestID, 16.0);
      rmSetAreaMaxBlobDistance(forestID, 25.0);
      rmSetAreaCoherence(forestID, 0.0);

      // Hill trees?
      rmSetAreaBaseHeight(forestID, rmRandFloat(1.0, 2.0));

      if(rmBuildArea(forestID)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;
   }

   int fishID=rmCreateObjectDef("fish");
   rmAddObjectDefItem(fishID, "fish - mahi", 3, 9.0);
   rmSetObjectDefMinDistance(fishID, 0.0);
   rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(1.5));
   rmAddObjectDefConstraint(fishID, fishVsFishID);
   rmAddObjectDefConstraint(fishID, fishLand);
   rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.5, 12*cNumberNonGaiaPlayers);

   // Text
   rmSetStatusText("",0.90);

   fishID=rmCreateObjectDef("fish2");
   rmAddObjectDefItem(fishID, "fish - mahi", 1, 0.0);
   rmSetObjectDefMinDistance(fishID, 0.0);
   rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(1.5));
   rmAddObjectDefConstraint(fishID, fishVsFishID);
   rmAddObjectDefConstraint(fishID, fishFurther);
   rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.5, 8*cNumberNonGaiaPlayers);

   // Random trees.
   rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 25*cNumberNonGaiaPlayers);
   
   // Rocks
   int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);
   int rockID=rmCreateObjectDef("rock");
   rmAddObjectDefItem(rockID, "flowers", 1, 0.0);
   rmSetObjectDefMinDistance(rockID, 0.0);
   rmSetObjectDefMaxDistance(rockID, rmXFractionToMeters(0.5));
   //rmAddObjectDefConstraint(rockID, avoidRock);
   rmAddObjectDefConstraint(rockID, avoidAll);
   rmAddObjectDefConstraint(rockID, avoidImpassableLand);
   rmAddAreaConstraint(rockID, rockaway);
   rmPlaceObjectDefAtLoc(rockID, 0, 0.5, 0.5, 30*cNumberNonGaiaPlayers);


  // Text
   rmSetStatusText("",1.0);


}  
