/*	Map Name: Sea of Worms.xs
**	Fast-Paced Ruleset: Mediterranean.xs
**	Author: Milkman Matty
**	Made for Forgotten Empires.
*/

//Include the library file.
include "MmM_FE_lib.xs";

// Main entry point for random map script
void main(void){

// Define triggers

for(i=1; <cNumberPlayers)
{
   rmCreateTrigger("FarmDebuff"+i);
}


for(i=1; <cNumberPlayers)
{
   rmSwitchToTrigger(rmTriggerID("FarmDebuff"+i));
   rmSetTriggerActive(true);

   rmAddTriggerEffect("Modify Protounit");
   code("trModifyProtounit(\"Farm\", "+i+", 4, 15);");
}
	
	/* **************************** */
	/* Section 1 Map Initialization */
	/* **************************** */
	
	// Create loading bar.
	rmSetStatusText("",0.01);
	
	int mapSizeMultiplier = 1;
	int playerTiles=9750;
	if(cMapSize == 1) {
		playerTiles = 12700;
		rmEchoInfo("Large map");
	} else if(cMapSize == 2){
		playerTiles = 25400;
		rmEchoInfo("Giant map");
		mapSizeMultiplier = 2;
	}
	int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
	rmEchoInfo("Map size="+size+"m x "+size+"m");
	rmSetMapSize(size, size);
	rmSetSeaLevel(0.0);
	
	rmSetSeaType("Marsh Pool");
	rmTerrainInitialize("MarshF");
    rmSetGaiaCiv(cCivOdin);
	
    rmSetLightingSet("night");
	rmSetStatusText("",0.07);
	
	/* ***************** */
	/* Section 2 Classes */
	/* ***************** */
	
	int classForest=rmDefineClass("forest");
	int classPlayer=rmDefineClass("player");
	int classCenter=rmDefineClass("center");
	int classPlayerCore=rmDefineClass("player core");
	int classpatch=rmDefineClass("patch");
	int classBonusIsland=rmDefineClass("bonus island");
	int classStartingSettlement = rmDefineClass("starting settlement");
	
	rmSetStatusText("",0.13);
	
	/* **************************** */
	/* Section 3 Global Constraints */
	/* **************************** */
	// For Areas and Objects. Local Constraints should be defined right before they are used.
	
	int playerConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, size*0.05);
	int centerConstraint=rmCreateClassDistanceConstraint("stay away from center", rmClassID("center"), 15.0);
	int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "land", false, 10.0);
	
	rmSetStatusText("",0.20);
	
	/* ********************* */
	/* Section 4 Map Outline */
	/* ********************* */
	
	int hand=0;
	if(rmRandFloat(0,1)<0.5) {
		hand = 1;
	}
	rmEchoInfo("hand ="+hand);
	
	if(hand==1) {
		rmSetPlacementSection(0.55, 0.15);
	} else {
		rmSetPlacementSection(0.05, 0.65);
	}
	
	rmPlacePlayersCircular(0.42, 0.43, rmDegreesToRadians(5.0));
	rmRecordPlayerLocations();
	
	int centerID=rmCreateArea("center");
	rmSetAreaSize(centerID, 0.1, 0.1);
	if(hand==1) {
		rmSetAreaLocation(centerID, 0.6, 0.4);
	} else {
		rmSetAreaLocation(centerID, 0.4, 0.6);
	}
	rmSetAreaWaterType(centerID, "marsh pool");
	rmAddAreaToClass(centerID, rmClassID("center"));
	rmSetAreaBaseHeight(centerID, 0.0);
	rmSetAreaMinBlobs(centerID, 12*mapSizeMultiplier);
	rmSetAreaMaxBlobs(centerID, 15*mapSizeMultiplier);
	rmSetAreaMinBlobDistance(centerID, 20);
	rmSetAreaMaxBlobDistance(centerID, 30*mapSizeMultiplier);
	rmSetAreaSmoothDistance(centerID, 50);
	rmSetAreaCoherence(centerID, 0.25);
	rmBuildArea(centerID);
	
	int gulfID=rmCreateArea("gulf");
	rmSetAreaSize(gulfID, 0.25, 0.25);
	if(hand==1) { 
		rmSetAreaLocation(gulfID, 0.9, 0.1);
	} else {
		rmSetAreaLocation(gulfID, 0.1, 0.9);
	}
	rmSetAreaWaterType(gulfID, "marsh pool");
	rmAddAreaToClass(gulfID, rmClassID("center"));
	rmSetAreaBaseHeight(gulfID, 0.0);
	rmSetAreaMinBlobs(gulfID, 8*mapSizeMultiplier);
	rmSetAreaMaxBlobs(gulfID, 10*mapSizeMultiplier);
	rmSetAreaMinBlobDistance(gulfID, 10);
	rmSetAreaMaxBlobDistance(gulfID, 20*mapSizeMultiplier);
	rmSetAreaSmoothDistance(gulfID, 50);
	rmSetAreaCoherence(gulfID, 0.25);
	rmBuildArea(gulfID);
	
	// Make bonus islands
	int numIsland = rmRandInt(2,3)+cNumberNonGaiaPlayers;
	if (numIsland>6) { 
		numIsland = 6;
	}
	
	int avoidLand=rmCreateTerrainDistanceConstraint("avoid land", "land", true, 25.0);
	
	for(i=1; <numIsland){
		int bonusID=rmCreateArea("Island"+i);
		rmSetAreaSize(bonusID, rmAreaTilesToFraction(300*mapSizeMultiplier), rmAreaTilesToFraction(500*mapSizeMultiplier));
		rmSetAreaMinBlobs(bonusID, 1*mapSizeMultiplier);
		rmSetAreaMaxBlobs(bonusID, 4*mapSizeMultiplier);
		rmSetAreaMinBlobDistance(bonusID, 20.0);
		rmSetAreaMaxBlobDistance(bonusID, 30.0*mapSizeMultiplier);
		rmSetAreaSmoothDistance(bonusID, 20);
		rmSetAreaCoherence(bonusID, 0.20);
		rmSetAreaBaseHeight(bonusID, 2.0);
		rmSetAreaHeightBlend(bonusID, 3);
		rmAddAreaToClass(bonusID, classBonusIsland);
		rmAddAreaConstraint(bonusID, avoidLand);
		rmSetAreaTerrainType(bonusID, "MarshF");
		rmAddAreaTerrainLayer(bonusID, "MarshD", 0, 3);
	}
	
	rmSetStatusText("",0.26);
	
	/* ********************** */
	/* Section 5 Player Areas */
	/* ********************** */
	
	float playerFraction=rmAreaTilesToFraction(900);
	for(i=1; <cNumberPlayers) {
		int id=rmCreateArea("Player"+i);
		rmSetPlayerArea(i, id);
		rmSetAreaSize(id, 0.9*playerFraction, 1.1*playerFraction);
		rmAddAreaToClass(id, classPlayer);
		rmSetAreaMinBlobs(id, 4);
		rmSetAreaMaxBlobs(id, 5);
		rmSetAreaWarnFailure(id, false);
		rmSetAreaMinBlobDistance(id, 30.0);
		rmSetAreaMaxBlobDistance(id, 50.0);
		rmSetAreaSmoothDistance(id, 20);
		rmSetAreaCoherence(id, 0.20);
		rmSetAreaBaseHeight(id, 2.0);
		rmSetAreaHeightBlend(id, 2);
		rmAddAreaConstraint(id, playerConstraint);
		rmAddAreaConstraint(id, centerConstraint);
		rmSetAreaLocPlayer(id, i);
		rmSetAreaTerrainType(id, "MarshF");
		rmAddAreaTerrainLayer(id, "MarshE", 4, 6);
		rmAddAreaTerrainLayer(id, "MarshB", 2, 4);
		rmAddAreaTerrainLayer(id, "MarshD", 0, 2);
	}
	rmBuildAllAreas();
	
	for(i=1; <cNumberPlayers){
		id=rmCreateArea("cliff avoider"+i, rmAreaID("Player"+i));
		rmSetAreaSize(id, 0.01, 0.01);
		rmAddAreaToClass(id, classStartingSettlement);
		rmSetAreaCoherence(id, 1.0);
		rmSetAreaLocPlayer(id, i);
	}
	rmBuildAllAreas();
	
	// Loading bar 33% 
	rmSetStatusText("",0.33);
	
	/* *********************** */
	/* Section 6 Map Specifics */
	/* *********************** */
	
	int patchConstraint=rmCreateClassDistanceConstraint("patch v patch", rmClassID("patch"), 10.0);
	int patch = 0;
	int failCount=0;
	for(i=1; <cNumberPlayers*6*mapSizeMultiplier) {
		patch=rmCreateArea("patch"+i);
		rmSetAreaSize(patch, rmAreaTilesToFraction(50*mapSizeMultiplier), rmAreaTilesToFraction(100*mapSizeMultiplier));
		rmSetAreaWarnFailure(patch, false);
		rmSetAreaTerrainType(patch, "GrassB");
		rmSetAreaMinBlobs(patch, 3*mapSizeMultiplier);
		rmSetAreaMaxBlobs(patch, 5*mapSizeMultiplier);
		rmSetAreaMinBlobDistance(patch, 18.0);
		rmSetAreaMaxBlobDistance(patch, 45.0*mapSizeMultiplier);
		rmAddAreaToClass(patch, classpatch);
		rmAddAreaConstraint(patch, patchConstraint);
		rmAddAreaConstraint(patch, playerConstraint);
		rmAddAreaConstraint(patch, avoidImpassableLand);
		rmSetAreaCoherence(patch, 0.3);
		rmSetAreaSmoothDistance(patch, 8);
		
		if(rmBuildArea(patch)==false) {
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==3){
				break;
			}
		} else {
			failCount=0;
		}
		
	}
	
	int wideCenterConstraint=rmCreateClassDistanceConstraint("elevation avoids center", rmClassID("center"), 20.0);
	
	int numTries=11*cNumberNonGaiaPlayers;
	int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 20.0);
	failCount=0;
	for(i=0; <numTries*mapSizeMultiplier) {
		int elevID=rmCreateArea("elev"+i);
		rmSetAreaSize(elevID, rmAreaTilesToFraction(35*mapSizeMultiplier), rmAreaTilesToFraction(80*mapSizeMultiplier));
		rmSetAreaLocation(elevID, rmRandFloat(0.0, 1.0), rmRandFloat(0.0, 1.0));
		rmSetAreaWarnFailure(elevID, false);
		rmAddAreaConstraint(elevID, playerConstraint);
		rmAddAreaConstraint(elevID, wideCenterConstraint);
		if(rmRandFloat(0.0, 1.0)<0.5) {
			rmSetAreaTerrainType(elevID, "MarshF");
		}
		rmSetAreaBaseHeight(elevID, rmRandFloat(5.0, 7.0));
		rmSetAreaHeightBlend(elevID, 2);
		rmSetAreaMinBlobs(elevID, 1*mapSizeMultiplier);
		rmSetAreaMaxBlobs(elevID, 5*mapSizeMultiplier);
		rmSetAreaMinBlobDistance(elevID, 16.0);
		rmSetAreaMaxBlobDistance(elevID, 40.0*mapSizeMultiplier);
		rmSetAreaCoherence(elevID, 0.0);
		
		if(rmBuildArea(elevID)==false) {
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==10){
				break;
			}
		} else {
			failCount=0;
		}
	}
	
	numTries=9*cNumberNonGaiaPlayers;
	failCount=0;
	for(i=0; <numTries*mapSizeMultiplier) {
		elevID=rmCreateArea("wrinkle"+i);
		rmSetAreaSize(elevID, rmAreaTilesToFraction(45*mapSizeMultiplier), rmAreaTilesToFraction(60*mapSizeMultiplier));
		rmSetAreaLocation(elevID, rmRandFloat(0.0, 1.0), rmRandFloat(0.0, 1.0));
		rmSetAreaWarnFailure(elevID, false);
		rmSetAreaBaseHeight(elevID, rmRandFloat(3.0, 4.0));
		rmSetAreaHeightBlend(elevID, 1);
		rmSetAreaMinBlobs(elevID, 1*mapSizeMultiplier);
		rmSetAreaMaxBlobs(elevID, 3*mapSizeMultiplier);
		rmSetAreaMinBlobDistance(elevID, 16.0);
		rmSetAreaMaxBlobDistance(elevID, 20.0*mapSizeMultiplier);
		rmSetAreaCoherence(elevID, 0.0);
		rmAddAreaConstraint(elevID, playerConstraint);
		rmAddAreaConstraint(elevID, wideCenterConstraint);
		
		if(rmBuildArea(elevID)==false) {
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==10){
				break;
			}
		} else {
			failCount=0;
		}
	}
	
	rmSetStatusText("",0.40);
	
	/* **************************** */
	/* Section 7 Object Constraints */
	/* **************************** */
	// If a constraint is used in multiple sections then it is listed here.
	
	int edgeConstraint=rmCreateBoxConstraint("edge of map", rmXTilesToFraction(20), rmZTilesToFraction(20), 1.0-rmXTilesToFraction(20), 1.0-rmZTilesToFraction(20));
	int shortEdgeConstraint=rmCreateBoxConstraint("short edge of map", rmXTilesToFraction(8), rmZTilesToFraction(8), 1.0-rmXTilesToFraction(8), 1.0-rmZTilesToFraction(8));
	int shortAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by short distance", "AbstractSettlement", 20.0);
	int farStartingSettleConstraint=rmCreateClassDistanceConstraint("objects avoid player TCs", rmClassID("starting settlement"), 50.0);
	int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 30.0);
	int avoidFood=rmCreateTypeDistanceConstraint("avoid food", "food", 12.0);
	
	rmSetStatusText("",0.46);
	
	/* ********************************* */
	/* Section 8 Fair Location Placement */
	/* ********************************* */
	
	int startingGoldFairLocID = -1;
	if(rmRandFloat(0,1) > 0.5){
		startingGoldFairLocID = rmAddFairLoc("Starting Gold", true, false, 20, 21, 0, 15);
	} else {
		startingGoldFairLocID = rmAddFairLoc("Starting Gold", false, false, 20, 21, 0, 15);
	}
	if(rmPlaceFairLocs()){
		int startingGoldID=rmCreateObjectDef("Starting Gold");
		rmAddObjectDefItem(startingGoldID, "Gold Mine Small", 1, 0.0);
		for(i=1; <cNumberPlayers){
			for(j=0; <rmGetNumberFairLocs(i)){
				rmPlaceObjectDefAtLoc(startingGoldID, i, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j), 1);
			}
		}
	}
	rmResetFairLocs();
	
	int startingSettlementID=rmCreateObjectDef("starting settlement");
	rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
	rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
	rmSetObjectDefMinDistance(startingSettlementID, 0.0);
	rmSetObjectDefMaxDistance(startingSettlementID, 0.0);
	rmPlaceObjectDefPerPlayer(startingSettlementID, true);
	
	int closeID = -1;
	int farID = -1;
	
	int TCavoidSettlement = rmCreateTypeDistanceConstraint("TC avoid TC by long distance", "AbstractSettlement", 50.0);
	int TCavoidStart = rmCreateClassDistanceConstraint("TC avoid starting by long distance", classStartingSettlement, 30.0);
	int TCavoidWater = rmCreateTerrainDistanceConstraint("TC avoid water", "Water", true, 30.0);
	int TCavoidImpassableLand = rmCreateTerrainDistanceConstraint("TC avoid badlands", "land", false, 18.0);
	
	if(cNumberNonGaiaPlayers == 2){
		//New way to place TC's. Places them 1 at a time.
		//This way ensures that FairLocs (TC's) will never be too close.
		for(p = 1; <= cNumberNonGaiaPlayers){
			
			//Add a new FairLoc every time. This will have to be removed before the next FairLoc is created.
			id=rmAddFairLoc("Settlement", false, true, 55, 75, 0, 20);
			rmAddFairLocConstraint(id, TCavoidImpassableLand);
			rmAddFairLocConstraint(id, TCavoidSettlement);
			rmAddFairLocConstraint(id, TCavoidStart);
			
			if(rmPlaceFairLocs()) {
				id=rmCreateObjectDef("close settlement"+p);
				rmAddObjectDefItem(id, "Settlement", 1, 0.0);
				rmPlaceObjectDefAtLoc(id, p, rmFairLocXFraction(p, 0), rmFairLocZFraction(p, 0), 1);
				int settleArea = rmCreateArea("settlement area"+p);
				rmSetAreaLocation(settleArea, rmFairLocXFraction(p, 0), rmFairLocZFraction(p, 0));
				rmSetAreaSize(settleArea, 0.01, 0.01);
				rmSetAreaTerrainType(settleArea, "MarshF");
				rmAddAreaTerrainLayer(settleArea, "MarshE", 4, 6);
				rmAddAreaTerrainLayer(settleArea, "MarshB", 2, 4);
				rmAddAreaTerrainLayer(settleArea, "MarshD", 0, 2);
				rmBuildArea(settleArea);
			}
			//Remove the FairLoc that we just created
			rmResetFairLocs();
		
			//Do it again.
			//Add a new FairLoc every time. This will have to be removed at the end of the block.
			id=rmAddFairLoc("Settlement", true, false, 70, 100, 60, 10);
			rmAddFairLocConstraint(id, TCavoidSettlement);
			rmAddFairLocConstraint(id, TCavoidStart);
			rmAddFairLocConstraint(id, TCavoidWater);
			
			if(rmPlaceFairLocs()) {
				id=rmCreateObjectDef("far settlement"+p);
				rmAddObjectDefItem(id, "Settlement", 1, 0.0);
				rmPlaceObjectDefAtLoc(id, p, rmFairLocXFraction(p, 0), rmFairLocZFraction(p, 0), 1);
				int settlementArea = rmCreateArea("settlement_area_"+p);
				rmSetAreaLocation(settlementArea, rmFairLocXFraction(p, 0), rmFairLocZFraction(p, 0));
				rmSetAreaSize(settlementArea, 0.01, 0.01);
				rmSetAreaTerrainType(settlementArea, "MarshF");
				rmAddAreaTerrainLayer(settlementArea, "MarshE", 4, 6);
				rmAddAreaTerrainLayer(settlementArea, "MarshB", 2, 4);
				rmAddAreaTerrainLayer(settlementArea, "GrassB", 0, 2);
				rmBuildArea(settlementArea);
			}
			rmResetFairLocs();	//Reset the data so that the next player doesn't place an extra TC.
		}
	} else {
		TCavoidSettlement = rmCreateTypeDistanceConstraint("TC avoid TC by super long distance", "AbstractSettlement", 65.0);
		for(p = 1; <= cNumberNonGaiaPlayers){
		
			closeID=rmCreateObjectDef("close settlement"+p);
			rmAddObjectDefItem(closeID, "Settlement", 1, 0.0);
			rmAddObjectDefConstraint(closeID, shortEdgeConstraint);
			rmAddObjectDefConstraint(closeID, TCavoidSettlement);
			rmAddObjectDefConstraint(closeID, TCavoidStart);
			rmAddObjectDefConstraint(closeID, TCavoidWater);
			for(attempt = 4; < 17){
				rmPlaceObjectDefAtLoc(closeID, p, rmGetPlayerX(p), rmGetPlayerZ(p), 1);
				if(rmGetNumberUnitsPlaced(closeID) > 0){
					break;
				}
				rmSetObjectDefMaxDistance(closeID, 12*attempt);
			}
		
			farID=rmCreateObjectDef("far settlement"+p);
			rmAddObjectDefItem(farID, "Settlement", 1, 0.0);
			rmAddObjectDefConstraint(farID, shortEdgeConstraint);
			rmAddObjectDefConstraint(farID, TCavoidWater);
			rmAddObjectDefConstraint(farID, TCavoidStart);
			rmAddObjectDefConstraint(farID, TCavoidSettlement);
			for(attempt = 4; < 17){
				rmPlaceObjectDefAtLoc(farID, p, rmGetPlayerX(p), rmGetPlayerZ(p), 1);
				if(rmGetNumberUnitsPlaced(farID) > 0){
					break;
				}
				rmSetObjectDefMaxDistance(farID, 12*attempt);
			}
		}
	} rmResetFairLocs();
		
	if(cMapSize == 2){
		//And one last time if Giant.
		id=rmAddFairLoc("Settlement", false, false,  rmXFractionToMeters(0.26), rmXFractionToMeters(0.38), 70, 20);
		rmAddFairLocConstraint(id, TCavoidSettlement);
		rmAddFairLocConstraint(id, TCavoidStart);
		rmAddFairLocConstraint(id, TCavoidWater);
		
		if(cNumberNonGaiaPlayers == 2){
			id=rmAddFairLoc("Settlement", true, false,  rmXFractionToMeters(0.35), rmXFractionToMeters(0.45), 80, 20);
		} else {
			id=rmAddFairLoc("Settlement", true, false,  rmXFractionToMeters(0.23), rmXFractionToMeters(0.4), 70+(3*cNumberNonGaiaPlayers), 20);
		}
		rmAddFairLocConstraint(id, TCavoidSettlement);
		rmAddFairLocConstraint(id, TCavoidStart);
		rmAddFairLocConstraint(id, TCavoidWater);
		
		if(rmPlaceFairLocs()){
			for(p = 1; <= cNumberNonGaiaPlayers){
				for(FL = 0; < 2){
					id=rmCreateObjectDef("Giant settlement_"+p+"_"+FL);
					rmAddObjectDefItem(id, "Settlement", 1, 1.0);
					
					int settlementArea2 = rmCreateArea("other_settlement_area_"+p+"_"+FL);
					rmSetAreaLocation(settlementArea2, rmFairLocXFraction(p, FL), rmFairLocZFraction(p, FL));
					rmSetAreaSize(settlementArea2, 0.005, 0.005);
					rmSetAreaTerrainType(settlementArea2, "MarshF");
					rmAddAreaTerrainLayer(settlementArea2, "GrassB", 4, 6);
					rmAddAreaTerrainLayer(settlementArea2, "MarshE", 2, 4);
					rmAddAreaTerrainLayer(settlementArea2, "MarshB", 0, 2);
					rmBuildArea(settlementArea2);
					rmPlaceObjectDefAtAreaLoc(id, p, settlementArea2);
				}
			}
		} else {
			for(p = 1; <= cNumberNonGaiaPlayers){
				
				farID=rmCreateObjectDef("giant settlement"+p);
				rmAddObjectDefItem(farID, "Settlement", 1, 0.0);
				rmAddObjectDefConstraint(farID, shortEdgeConstraint);
				rmAddObjectDefConstraint(farID, TCavoidWater);
				rmAddObjectDefConstraint(farID, TCavoidStart);
				rmAddObjectDefConstraint(farID, TCavoidSettlement);
				for(attempt = 3; <= 7){
					rmPlaceObjectDefAtLoc(farID, p, rmGetPlayerX(p), rmGetPlayerZ(p), 1);
					if(rmGetNumberUnitsPlaced(farID) > 0){
						break;
					}
					rmSetObjectDefMaxDistance(farID, 25*attempt);
				}
				
				farID=rmCreateObjectDef("giant2 settlement"+p);
				rmAddObjectDefItem(farID, "Settlement", 1, 0.0);
				rmAddObjectDefConstraint(farID, shortEdgeConstraint);
				rmAddObjectDefConstraint(farID, TCavoidWater);
				rmAddObjectDefConstraint(farID, TCavoidStart);
				rmAddObjectDefConstraint(farID, TCavoidSettlement);
				for(attempt = 3; <= 7){
					rmPlaceObjectDefAtLoc(farID, p, rmGetPlayerX(p), rmGetPlayerZ(p), 1);
					if(rmGetNumberUnitsPlaced(farID) > 0){
						break;
					}
					rmSetObjectDefMaxDistance(farID, 25*attempt);
				}
			}
		}
	}
	
	rmResetFairLocs();
	
	int farAvoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land by lots", "land", false, 30.0);
	id = rmAddFairLoc("fortLand", true, true, 30, 35, 40, 16, true);
	rmAddFairLocConstraint(id, farAvoidImpassableLand);

	
	rmSetStatusText("",0.53);
	
	/* ************************** */
	/* Section 9 Starting Objects */
	/* ************************** */
	// Order of placement is Settlements then gold then food (hunt, herd, predator).
	
	int getOffTheTC = rmCreateTypeDistanceConstraint("Stop starting resources from somehow spawning on top of TC!", "AbstractSettlement", 16.0);
	
	int huntShortAvoidsStartingGoldMilky=rmCreateTypeDistanceConstraint("short hunty avoid gold", "gold", 10.0);
	int startingHuntableID=rmCreateObjectDef("starting hunt");
	rmAddObjectDefItem(startingHuntableID, "boar", rmRandInt(5,6), 5.0);
	rmSetObjectDefMaxDistance(startingHuntableID, 24.0);
	rmSetObjectDefMaxDistance(startingHuntableID, 27.0);
	rmAddObjectDefConstraint(startingHuntableID, huntShortAvoidsStartingGoldMilky);
	rmAddObjectDefConstraint(startingHuntableID, getOffTheTC);
	rmPlaceObjectDefPerPlayer(startingHuntableID, false, 1);
	
	int chickenShortAvoidsStartingGoldMilky=rmCreateTypeDistanceConstraint("short birdy avoid gold", "gold", 10.0);
	int startingChickenID=rmCreateObjectDef("starting birdies");
	rmAddObjectDefItem(startingChickenID, "Chicken", rmRandInt(7,10), 4.0);
	rmSetObjectDefMaxDistance(startingChickenID, 15.0);
	rmSetObjectDefMaxDistance(startingChickenID, 18.0);
	rmAddObjectDefConstraint(startingChickenID, chickenShortAvoidsStartingGoldMilky);
	rmAddObjectDefConstraint(startingChickenID, getOffTheTC);
	rmAddObjectDefConstraint(startingChickenID, avoidFood);
	rmPlaceObjectDefPerPlayer(startingChickenID, false);
	
	int stragglerTreeID=rmCreateObjectDef("straggler tree");
	rmAddObjectDefItem(stragglerTreeID, "pine", 1, 0.0);
	rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
	rmSetObjectDefMaxDistance(stragglerTreeID, 15.0);
	rmAddObjectDefConstraint(stragglerTreeID, rmCreateTypeDistanceConstraint("trees avoid all", "all", 4.0));
	rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(5,7));
	
	int fishVsFishID=rmCreateTypeDistanceConstraint("fish v fish", "fish", 13.0);
	int ghostVsghostID=rmCreateTypeDistanceConstraint("ghost v ghost", "hostile boats", 16.0);
	int fishLand = rmCreateTerrainDistanceConstraint("fish land", "land", true, 5.0);
	int fishID=rmCreateObjectDef("fish");
	rmAddObjectDefItem(fishID, "fish - perch", 3, 9.0);
	rmSetObjectDefMinDistance(fishID, 0.0);
	rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(fishID, fishVsFishID);
	rmAddObjectDefConstraint(fishID, fishLand);
	rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.5, 6*cNumberNonGaiaPlayers);
	
	fishID=rmCreateObjectDef("fish2");
	rmAddObjectDefItem(fishID, "fish - perch", 3, 7.0);
	rmSetObjectDefMinDistance(fishID, 0.0);
	rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(fishID, fishVsFishID);
	rmAddObjectDefConstraint(fishID, fishLand);
	rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.5, 4*cNumberNonGaiaPlayers);
	
	int lostBoatsID=rmCreateObjectDef("hostile boats");
	rmAddObjectDefItem(lostBoatsID, "Ghost Ship", 2, 1.0);
	rmSetObjectDefMinDistance(lostBoatsID, 0.5);
	rmSetObjectDefMaxDistance(lostBoatsID, rmXFractionToMeters(1.5));
	rmAddObjectDefConstraint(lostBoatsID, fishLand);
	rmPlaceObjectDefAtLoc(lostBoatsID, 0, 0.5, 0.5, 2*cNumberNonGaiaPlayers);
	
	rmSetStatusText("",0.60);
	
	/* *************************** */
	/* Section 10 Starting Forests */
	/* *************************** */
	
	int forestTerrain = rmCreateTerrainDistanceConstraint("forest terrain", "Land", false, 3.0);
	int forestTC = rmCreateClassDistanceConstraint("starting forest vs starting settle", classStartingSettlement, 20.0);
	int forestOtherTCs = rmCreateTypeDistanceConstraint("starting forest vs settle", "AbstractSettlement", 20.0);
	
	int maxNum = 4;
	for(p=1;<=cNumberNonGaiaPlayers){
		placePointsCircleCustom(rmXMetersToFraction(45.0), maxNum, -1.0, -1.0, rmGetPlayerX(p), rmGetPlayerZ(p), false, false);
		int skip = rmRandInt(1,maxNum);
		for(i=1; <= maxNum){
			if(i == skip){
				continue;
			}
			int playerStartingForestID=rmCreateArea("player "+p+" forest "+i);
			rmSetAreaSize(playerStartingForestID, rmAreaTilesToFraction(75+cNumberNonGaiaPlayers), rmAreaTilesToFraction(100+cNumberNonGaiaPlayers));
			rmSetAreaLocation(playerStartingForestID, rmGetCustomLocXForPlayer(i), rmGetCustomLocZForPlayer(i));
			rmSetAreaWarnFailure(playerStartingForestID, true);
			rmSetAreaForestType(playerStartingForestID, "pine forest");
			rmAddAreaConstraint(playerStartingForestID, forestOtherTCs);
			rmAddAreaConstraint(playerStartingForestID, forestTC);
			rmAddAreaConstraint(playerStartingForestID, forestTerrain);
			rmAddAreaToClass(playerStartingForestID, classForest);
			rmSetAreaCoherence(playerStartingForestID, 0.25);
			rmBuildArea(playerStartingForestID);
		}
	}
	
	rmSetStatusText("",0.66);
	
	/* ************************* */
	/* Section 11 Medium Objects */
	/* ************************* */
	// Order of placement is Settlements then gold then food (hunt, herd, predator).
	
	int mediumGoldID=rmCreateObjectDef("medium gold");
	rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
	rmSetObjectDefMinDistance(mediumGoldID, 55.0);
	rmSetObjectDefMaxDistance(mediumGoldID, 65.0);
	rmAddObjectDefConstraint(mediumGoldID, avoidGold);
	rmAddObjectDefConstraint(mediumGoldID, edgeConstraint);
	rmAddObjectDefConstraint(mediumGoldID, shortAvoidSettlement);
	rmAddObjectDefConstraint(mediumGoldID, avoidImpassableLand);
	rmAddObjectDefConstraint(mediumGoldID, farStartingSettleConstraint);
	rmPlaceObjectDefPerPlayer(mediumGoldID, false);
	
	int mediumHuntID=rmCreateObjectDef("medium hunt");
	if(rmRandInt(0,1) == 0){
		rmAddObjectDefItem(mediumHuntID, "boar", rmRandInt(4,5), 3.0);
	} else {
		rmAddObjectDefItem(mediumHuntID, "boar", rmRandInt(4,5)+1, 3.0);
	}
	rmSetObjectDefMaxDistance(mediumHuntID, 45.0);
	rmSetObjectDefMaxDistance(mediumHuntID, 50.0);
	rmAddObjectDefConstraint(mediumHuntID, avoidFood);
	rmAddObjectDefConstraint(mediumHuntID, avoidGold);
	rmAddObjectDefConstraint(mediumHuntID, shortAvoidSettlement);
	rmAddObjectDefConstraint(mediumHuntID, centerConstraint);
	rmAddObjectDefConstraint(mediumHuntID, farStartingSettleConstraint);
	rmPlaceObjectDefPerPlayer(mediumHuntID, false, 1);
	
	rmSetStatusText("",0.73);
	
	/* ********************** */
	/* Section 12 Far Objects */
	/* ********************** */
	// Order of placement is Settlements then gold then food (hunt, herd, predator).
	
	int farGoldID=rmCreateObjectDef("far gold");
	rmAddObjectDefItem(farGoldID, "Gold mine", 1, 0.0);
	rmSetObjectDefMinDistance(farGoldID, 70.0);
	rmSetObjectDefMaxDistance(farGoldID, 110.0);
	rmAddObjectDefConstraint(farGoldID, avoidGold);
	rmAddObjectDefConstraint(farGoldID, avoidImpassableLand);
	rmAddObjectDefConstraint(farGoldID, shortAvoidSettlement);
	rmAddObjectDefConstraint(farGoldID, farStartingSettleConstraint);
	rmPlaceObjectDefPerPlayer(farGoldID, false, rmRandInt(1,2));
	
	int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("short avoid impassable land", "land", false, 4.0);
	
	int bonusGoldID=rmCreateObjectDef("bonus gold");
	rmAddObjectDefItem(bonusGoldID, "pine", 3, 0.4);
	rmSetObjectDefMinDistance(bonusGoldID, 0.0);
	rmSetObjectDefMaxDistance(bonusGoldID, 1.0);
	rmAddObjectDefConstraint(bonusGoldID, shortAvoidImpassableLand);
	rmAddObjectDefConstraint(bonusGoldID, shortAvoidSettlement);
	rmAddObjectDefConstraint(bonusGoldID, farStartingSettleConstraint);
	for(i=1; <numIsland*12) {
		rmPlaceObjectDefInRandomAreaOfClass(bonusGoldID, 0, classBonusIsland);
	}
	
	int pineStumpID=rmCreateObjectDef("pine stumper");
	rmAddObjectDefItem(pineStumpID, "stump pine", 1, 0.4);
	rmSetObjectDefMinDistance(pineStumpID, 0.0);
	rmSetObjectDefMaxDistance(pineStumpID, 1.0);
	rmAddObjectDefConstraint(pineStumpID, shortAvoidImpassableLand);
	rmAddObjectDefConstraint(pineStumpID, shortAvoidSettlement);
	rmAddObjectDefConstraint(pineStumpID, farStartingSettleConstraint);
	for(i=1; <numIsland*5) {
		rmPlaceObjectDefInRandomAreaOfClass(pineStumpID, 0, classBonusIsland);
	}
	
	int bonusRelic2ID=rmCreateObjectDef("island relic");
	int randRelicFactor=rmRandFloat(0,1);
	if(randRelicFactor>0.8){
	randRelicFactor = 2;
	} else {
	randRelicFactor = 1;
	}
	rmAddObjectDefItem(bonusRelic2ID, "Relic", randRelicFactor, 0.4);
	rmSetObjectDefMinDistance(bonusRelic2ID, 0.0);
	rmSetObjectDefMaxDistance(bonusRelic2ID, 1.0);
	rmAddObjectDefConstraint(bonusRelic2ID, shortAvoidImpassableLand);
	rmAddObjectDefConstraint(bonusRelic2ID, shortAvoidSettlement);
	rmAddObjectDefConstraint(bonusRelic2ID, farStartingSettleConstraint);
	for(i=1; <numIsland) {
		rmPlaceObjectDefInRandomAreaOfClass(bonusRelic2ID, 0, classBonusIsland);
	}
	
	int randHouseID=rmCreateObjectDef("island house");
	rmAddObjectDefItem(randHouseID, "house", 1, 0.4);
	rmSetObjectDefMinDistance(randHouseID, 0.0);
	rmSetObjectDefMaxDistance(randHouseID, 1.0);
	rmAddObjectDefConstraint(randHouseID, shortAvoidImpassableLand);
	rmAddObjectDefConstraint(randHouseID, TCavoidSettlement);
	rmAddObjectDefConstraint(randHouseID, farStartingSettleConstraint);
	for(i=1; <numIsland) {
		rmPlaceObjectDefInRandomAreaOfClass(randHouseID, 0, classBonusIsland);
	}
	
	int avoidHuntable=rmCreateTypeDistanceConstraint("avoid huntable", "huntable", 35.0);
	int bonusHuntableID=rmCreateObjectDef("bonus huntable");
	float bonusChance=rmRandFloat(0, 1);
	if(bonusChance<0.5) {
		rmAddObjectDefItem(bonusHuntableID, "boar", rmRandInt(1,3), 4.0);
	} else {
		rmAddObjectDefItem(bonusHuntableID, "boar", rmRandInt(1,4), 5.0);
	}
	rmSetObjectDefMinDistance(bonusHuntableID, 65.0);
	rmSetObjectDefMaxDistance(bonusHuntableID, 75.0+(cNumberNonGaiaPlayers*2-4));
	rmAddObjectDefConstraint(bonusHuntableID, avoidHuntable);
	rmAddObjectDefConstraint(bonusHuntableID, avoidGold);
	rmAddObjectDefConstraint(bonusHuntableID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(bonusHuntableID, avoidImpassableLand);
	rmPlaceObjectDefPerPlayer(bonusHuntableID, false);
	
	int farPredatorID=rmCreateObjectDef("far predator");
	float predatorSpecies=rmRandFloat(0, 1);
	if(predatorSpecies<0.5) {
		rmAddObjectDefItem(farPredatorID, "bear", 2, 4.0);
	} else {
		rmAddObjectDefItem(farPredatorID, "bear", 2, 6.0);
	}
	rmSetObjectDefMinDistance(farPredatorID, 70.0);
	rmSetObjectDefMaxDistance(farPredatorID, 90.0);
	rmAddObjectDefConstraint(farPredatorID, rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 30.0));
	rmAddObjectDefConstraint(farPredatorID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(farPredatorID, rmCreateTypeDistanceConstraint("preds avoid foody 93", "food", 25.0));
	rmAddObjectDefConstraint(farPredatorID, rmCreateTerrainDistanceConstraint("preds avoid impassable land", "land", false, 3.0));
	rmAddObjectDefConstraint(farPredatorID, rmCreateTypeDistanceConstraint("preds avoid gold 93", "gold", 35.0));
	rmAddObjectDefConstraint(farPredatorID, rmCreateTypeDistanceConstraint("preds avoid settlements 93", "AbstractSettlement", 40.0));
	rmPlaceObjectDefPerPlayer(farPredatorID, false, 1);
	
	int relicID=rmCreateObjectDef("relic");
	rmAddObjectDefItem(relicID, "relic", rmRandInt(0,1), 0.0);
	rmSetObjectDefMinDistance(relicID, 75.0);
	rmSetObjectDefMaxDistance(relicID, 110.0);
	rmAddObjectDefConstraint(relicID, edgeConstraint);
	rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 70.0));
	rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic avoid gold 103", "gold", 20.0));
	rmAddObjectDefConstraint(relicID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(relicID, shortAvoidSettlement);
	rmAddObjectDefConstraint(relicID, avoidImpassableLand);
	rmPlaceObjectDefPerPlayer(relicID, false);
	
	int randomTreeID=rmCreateObjectDef("random tree");
	rmAddObjectDefItem(randomTreeID, "pine", 1, 0.0);
	rmSetObjectDefMinDistance(randomTreeID, 0.0);
	rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 6.0));
	rmAddObjectDefConstraint(randomTreeID, shortAvoidSettlement);
	rmAddObjectDefConstraint(randomTreeID, avoidImpassableLand);
	rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 13*cNumberNonGaiaPlayers);
	
	int randomHouseID=rmCreateObjectDef("random house");
	rmAddObjectDefItem(randomHouseID, "house", 1, 0.0);
	rmSetObjectDefMinDistance(randomHouseID, 0.0);
	rmSetObjectDefMaxDistance(randomHouseID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(randomHouseID, rmCreateTypeDistanceConstraint("random tree", "all", 3.0));
	rmAddObjectDefConstraint(randomHouseID, shortAvoidSettlement);
	rmAddObjectDefConstraint(randomHouseID, avoidImpassableLand);
	rmPlaceObjectDefAtLoc(randomHouseID, 0, 0.5, 0.5, 2*cNumberNonGaiaPlayers);
	
	rmSetStatusText("",0.80);
	
	/* ************************ */
	/* Section 13 Giant Objects */
	/* ************************ */

	if(cMapSize == 2){
		int giantGoldID=rmCreateObjectDef("giant gold");
		rmAddObjectDefItem(giantGoldID, "Gold mine", 1, 0.0);
		rmSetObjectDefMaxDistance(giantGoldID, rmXFractionToMeters(0.28));
		rmSetObjectDefMaxDistance(giantGoldID, rmXFractionToMeters(0.38));
		rmAddObjectDefConstraint(giantGoldID, avoidFood);
		rmAddObjectDefConstraint(giantGoldID, shortAvoidSettlement);
		rmAddObjectDefConstraint(giantGoldID, rmCreateTypeDistanceConstraint("gold avoid gold 93", "gold", 45.0));
		rmPlaceObjectDefPerPlayer(giantGoldID, false, 3);
		
		int giantHuntableID=rmCreateObjectDef("giant huntable");
		rmAddObjectDefItem(giantHuntableID, "huntable", rmRandInt(5,7), 5.0);
		rmSetObjectDefMaxDistance(giantHuntableID, rmXFractionToMeters(0.3));
		rmSetObjectDefMaxDistance(giantHuntableID, rmXFractionToMeters(0.36));
		rmAddObjectDefConstraint(giantHuntableID, avoidHuntable);
		rmAddObjectDefConstraint(giantHuntableID, avoidFood);
		rmAddObjectDefConstraint(giantHuntableID, avoidGold);
		rmAddObjectDefConstraint(giantHuntableID, shortAvoidSettlement);
		rmAddObjectDefConstraint(giantHuntableID, farStartingSettleConstraint);
		rmAddObjectDefConstraint(giantHuntableID, avoidImpassableLand);
		rmPlaceObjectDefPerPlayer(giantHuntableID, false, rmRandInt(1, 2));
		
		int giantRelixID = rmCreateObjectDef("giant Relix");
		rmAddObjectDefItem(giantRelixID, "relic", 1, 5.0);
		rmSetObjectDefMaxDistance(giantRelixID, rmXFractionToMeters(0.0));
		rmSetObjectDefMaxDistance(giantRelixID, rmXFractionToMeters(0.5));
		rmAddObjectDefConstraint(giantRelixID, avoidGold);
		rmAddObjectDefConstraint(giantRelixID, shortAvoidSettlement);
		rmAddObjectDefConstraint(giantRelixID, rmCreateTypeDistanceConstraint("relix avoid relix", "relic", 110.0));
		rmPlaceObjectDefAtLoc(giantRelixID, 0, 0.5, 0.5, cNumberNonGaiaPlayers);
	}
	
	rmSetStatusText("",0.86);
	
	/* ************************************ */
	/* Section 14 Map Fill Cliffs & Forests */
	/* ************************************ */
	
	int forestObjConstraint=rmCreateTypeDistanceConstraint("forest obj", "all", 6.0);
	int forestConstraint=rmCreateClassDistanceConstraint("forest v forest", rmClassID("forest"), 10.0);
	int forestSettleConstraint=rmCreateTypeDistanceConstraint("forest settle", "AbstractSettlement", 31.0);
	int forestCount=15*cNumberNonGaiaPlayers;
	if(cMapSize == 2){
		forestCount = 1.25*forestCount;
	}
	
	failCount=0;
	for(i=0; <forestCount) {
		int forestID=rmCreateArea("forest"+i);
		rmSetAreaSize(forestID, rmAreaTilesToFraction(55), rmAreaTilesToFraction(80));
		if(cMapSize == 2){
			rmSetAreaSize(forestID, rmAreaTilesToFraction(75), rmAreaTilesToFraction(120));
		}
		
		
		rmSetAreaForestType(forestID, "pine forest");
		rmAddAreaConstraint(forestID, forestSettleConstraint);
		rmAddAreaConstraint(forestID, forestObjConstraint);
		rmAddAreaConstraint(forestID, forestConstraint);
		rmAddAreaConstraint(forestID, avoidImpassableLand);
		rmAddAreaToClass(forestID, classForest);
		
		rmSetAreaMinBlobs(forestID, 1);
		rmSetAreaMaxBlobs(forestID, 5);
		rmSetAreaMinBlobDistance(forestID, 16.0);
		rmSetAreaMaxBlobDistance(forestID, 40.0);
		rmSetAreaCoherence(forestID, 0.0);
		
		// Hill trees?
		if(rmRandFloat(0.0, 1.0)<0.2) {
			rmSetAreaBaseHeight(forestID, rmRandFloat(3.0, 4.0));
		}
		
		if(rmBuildArea(forestID)==false) {
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==3*mapSizeMultiplier) {
				break;
			}
		} else {
			failCount=0;
		}
	}
	
	rmSetStatusText("",0.93);
	
	/* ********************************* */
	/* Section 15 Beautification Objects */
	/* ********************************* */
	// Placed in no particular order.
	
	int farhawkID=rmCreateObjectDef("far hawks");
	rmAddObjectDefItem(farhawkID, "hawk", 1, 0.0);
	rmSetObjectDefMinDistance(farhawkID, 0.0);
	rmSetObjectDefMaxDistance(farhawkID, rmXFractionToMeters(0.5));
	rmPlaceObjectDefPerPlayer(farhawkID, false, 2);
	
	int sharkLand = rmCreateTerrainDistanceConstraint("shark land", "land", true, 20.0);
	int sharkVssharkID=rmCreateTypeDistanceConstraint("shark v shark", "shark", 20.0);
	int sharkVssharkID2=rmCreateTypeDistanceConstraint("shark v orca", "orca", 20.0);
	int sharkVssharkID3=rmCreateTypeDistanceConstraint("shark v whale", "whale", 20.0);
	
	int sharkID=rmCreateObjectDef("shark");
	if(rmRandFloat(0,1)<0.5) {
		rmAddObjectDefItem(sharkID, "pine", 1, 0.0);
	} else {
		rmAddObjectDefItem(sharkID, "seaweed", 1, 0.0);
	}
	rmSetObjectDefMinDistance(sharkID, 0.0);
	rmSetObjectDefMaxDistance(sharkID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(sharkID, edgeConstraint);
	rmAddObjectDefConstraint(sharkID, sharkVssharkID);
	rmAddObjectDefConstraint(sharkID, sharkVssharkID2);
	rmAddObjectDefConstraint(sharkID, sharkVssharkID3);
	rmAddObjectDefConstraint(sharkID, sharkLand);
	rmPlaceObjectDefAtLoc(sharkID, 0, 0.5, 0.5, cNumberNonGaiaPlayers*15);
	
	int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);
	int avoidGrass=rmCreateTypeDistanceConstraint("avoid grass", "grass", 12.0);
	int grassID=rmCreateObjectDef("grass");
	
	rmAddObjectDefItem(grassID, "grass", 3, 4.0);
	rmSetObjectDefMinDistance(grassID, 0.0);
	rmSetObjectDefMaxDistance(grassID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(grassID, avoidGrass);
	rmAddObjectDefConstraint(grassID, avoidAll);
	rmAddObjectDefConstraint(grassID, avoidImpassableLand);
	rmPlaceObjectDefAtLoc(grassID, 0, 0.5, 0.5, 35*cNumberNonGaiaPlayers);
	
	int rockID=rmCreateObjectDef("rock and grass");
	int avoidRock=rmCreateTypeDistanceConstraint("avoid rock", "rock limestone sprite", 8.0);
	rmAddObjectDefItem(rockID, "rock limestone sprite", 1, 1.0);
	rmAddObjectDefItem(rockID, "grass", 2, 1.0);
	rmSetObjectDefMinDistance(rockID, 0.0);
	rmSetObjectDefMaxDistance(rockID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(rockID, avoidAll);
	rmAddObjectDefConstraint(rockID, avoidImpassableLand);
	rmAddObjectDefConstraint(rockID, avoidRock);
	rmPlaceObjectDefAtLoc(rockID, 0, 0.5, 0.5, 15*cNumberNonGaiaPlayers);
	
	int rockID2=rmCreateObjectDef("rock group");
	rmAddObjectDefItem(rockID2, "rock limestone sprite", 3, 2.0);
	rmSetObjectDefMinDistance(rockID2, 0.0);
	rmSetObjectDefMaxDistance(rockID2, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(rockID2, avoidAll);
	rmAddObjectDefConstraint(rockID2, avoidImpassableLand);
	rmAddObjectDefConstraint(rockID2, avoidRock);
	rmPlaceObjectDefAtLoc(rockID2, 0, 0.5, 0.5, 8*cNumberNonGaiaPlayers);
	
   
   int pestilenceAmbient1=rmCreateObjectDef("pestilence ambience 1");
   rmAddObjectDefItem(pestilenceAmbient1, "Pestilence SFX1", 1, 1.0);
   rmSetObjectDefMinDistance(pestilenceAmbient1, 0.5);
   rmSetObjectDefMaxDistance(pestilenceAmbient1, rmXFractionToMeters(1.0));
   rmPlaceObjectDefAtLoc(pestilenceAmbient1, 0, 0.5, 0.5, 7*cNumberNonGaiaPlayers);
   
   int pestilenceAmbient2=rmCreateObjectDef("pestilence ambience 2");
   rmAddObjectDefItem(pestilenceAmbient2, "Pestilence SFX2", 1, 1.0);
   rmSetObjectDefMinDistance(pestilenceAmbient2, 0.5);
   rmSetObjectDefMaxDistance(pestilenceAmbient2, rmXFractionToMeters(1.0));
   rmPlaceObjectDefAtLoc(pestilenceAmbient2, 0, 0.5, 0.5, 7*cNumberNonGaiaPlayers);
   
   int rngdgold=rmCreateObjectDef("jesus gold");
   rmAddObjectDefItem(rngdgold, "gold mine small", 1, 1.0);
   rmSetObjectDefMinDistance(rngdgold, 0.5);
   rmSetObjectDefMaxDistance(rngdgold, rmXFractionToMeters(1.0));
   rmAddObjectDefConstraint(rngdgold, avoidImpassableLand);
   rmAddObjectDefConstraint(rngdgold, farStartingSettleConstraint);
   rmPlaceObjectDefAtLoc(rngdgold, 0, 0.5, 0.5, 1*cNumberNonGaiaPlayers);
   
   int logsID=rmCreateObjectDef("random logs");
   rmAddObjectDefItem(logsID, "rotting log", 1, 1.0);
   rmSetObjectDefMinDistance(logsID, 0.5);
   rmSetObjectDefMaxDistance(logsID, rmXFractionToMeters(1.0));
   rmPlaceObjectDefAtLoc(logsID, 0, 0.5, 0.5, 7*cNumberNonGaiaPlayers);
	
	rmSetStatusText("",1.0);
}